# -*- mode: python; mode: font-lock; py-indent-offset: 4; -*-
# $Id: janitor.py,v 1.20 2004/10/17 20:27:17 mok Exp $
#
# Copyright (C) 2004 Morten Kjeldgaard <mok@bioxray.dk>
#
# This file is part of the program "repo-janitor".
#
# "repo-janitor" is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# "repo-janitor" is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with "repo-janitor" in a file named "COPYING".  If not, visit
# their website at http://www.gnu.org for a copy, or request a copy by
# writing to:
#     Free Software Foundation, Inc.
#     59 Temple Place, Suite 330
#     Boston, MA 02111-1307 USA

import os, sys, string
import xml.sax._exceptions
from parser import ConfigParser, make_parser
import message, buildenv
import webgen
from version import __version__

def run():
    from getopt import getopt
    from parser import gRepo


    C = ["check", "showconfig", "destroy", "create", "mirror", "generate",
         "purge", "clean", "webgen", "build", "quiet", "audit", "zap"]

    do_delete = False        # if True, certain commands _will_ delete files

    m = message.Message()
    parser = make_parser()
    curHandler = ConfigParser()
    parser.setContentHandler(curHandler)

    # parse options
    prog = os.path.basename(sys.argv[0])
    opts, args = getopt(sys.argv[1:],"VHDqT",
                        ["delete","version","help","html","quiet","template"
                         "debug"])

    for opt in opts:
        if opt[0] in ["-V", "--version"]:
            sys.stderr.write("%s %s\n" % (prog,__version__))
            raise SystemExit

        elif opt[0] in ["-q", "--quiet"]:
            message.Message.verbosity = message.Message.verbosity-1

        elif opt[0] in ["-H", "--html"]:
            message.Message.html = True

        elif opt[0] in ["-T", "--template"]:
            webgen.template = 1;

        elif opt[0] in ["-D", "--debug"]:
            message.Message.debug_mode = True

        elif opt[0] == "--help":
            sys.stderr.write("Usage: %s <config> [[command]...]\n" % prog)            
            sys.stderr.write("where command is one or more of: "+string.join(C,' '))
            sys.stderr.write('\n')
            raise SystemExit

        elif opt[0] == "--delete":
            do_delete = True
        #;
    #;

    if len(args) == 0:
        sys.stderr.write("Usage: %s <config> [[command]...]\n" % prog)
        raise SystemExit
    #;

    if not os.path.exists(args[0]):
        sys.stderr.write("config file %s does not exist\n" % args[0])
        raise SystemExit
    #;

    # Parse the XML configuration file...
    try:
        parser.parse(open(args[0]))
    except xml.sax._exceptions.SAXParseException, e:
        s = "%s: %s, line %d column %d\n" % (args[0],
                                            e.getMessage(),
                                            e.getLineNumber(),
                                            e.getColumnNumber())
        m.failure(s)
        raise SystemExit
    #;
    
    # All files we create must be world readable
    os.umask (022)

    # This is the main command loop. A command is passed down a chain of
    # responsibility, until it meets an object that wants to deal with it.

    repo = gRepo[0]
    
    for cmd in args[1:]:

        result = 0
        if cmd == C[0]:                 # check
            result = repo.check(0)

        elif cmd == C[1]:               # showconfig
            repo.info()
            
        elif cmd == C[2]:               # destroy
            repo.clean(True)
            repo.destroy()

        elif cmd == C[3]:               # create
            repo.create()

        elif cmd == C[4]:               # mirror
            repo.mirror()

        elif cmd == C[5]:               # generate
            result = repo.generate()
            if result == 0:
                m.warning("ALL YOUR BASE ARE BELONG TO US!\n\n")
            else:
                m.failure("SOMEBODY SET UP US THE BOMB!\n\n")
            #;

        elif cmd == C[6]:               # purge
            repo.purge(do_delete)

        elif cmd == C[7]:               # clean
            repo.clean(do_delete)

        elif cmd == C[8]:               # webgen
            repo.webgen(webgen.Document())

        elif cmd == C[9]:               # build
            be = buildenv.BuildEnv()
            be.create()
            repo.build()
            be.destroy()
            
        elif cmd == C[10]:              # quiet
            message.Message.verbosity = message.Message.verbosity-1

        elif cmd == C[11]:              # audit
            result = repo.audit(do_delete)
            #;

        elif cmd == C[12]:              # zap
            result = repo.zap()
            #;

        else:
            m.failure("unknown command \"%s\"\n" % cmd)
            m.info ("valid commands are:\n")
            m.info(string.join(C,' ')+'\n')
            raise SystemExit
        #;
        if result == 0:
            m.debug("OK command %s\n" % cmd)
        elif result == 1:
            m.debug("WARNING command %s\n" % cmd)
        elif result == 2:
            m.debug("FAILURE command %s\n" % cmd)            
        #;
    #;  end of command loop
    return
#;


if __name__ == "__main__":
    run()
#;

####
