# -*- mode: python; mode: font-lock; py-indent-offset: 4; -*-
# $Id: message.py,v 1.3 2004/08/19 19:17:26 mok Exp $
#
# Copyright (C) 2004 Morten Kjeldgaard <mok@bioxray.dk>
#
# This file is part of the program "repo-janitor".
#
# "repo-janitor" is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# "repo-janitor" is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with "repo-janitor" in a file named "COPYING".  If not, visit
# their website at http://www.gnu.org for a copy, or request a copy by
# writing to:
#     Free Software Foundation, Inc.
#     59 Temple Place, Suite 330
#     Boston, MA 02111-1307 USA

import sys

# Class to handle output and error messages.  The idea of the nicely
# colored prompts were ruthlessly stolen from RedHat's
# /etc/init.d/functions ;-)

class Message:
    html = False
    verbosity = 3                       # class variables
    indent = 0
    debug_mode = False
    last = -1
    
    def failure(self, str):
        self.last = 0
        if self.verbosity > self.last:
            if self.html:
                sys.stdout.write("<tt>[<font color=\"#ff0000\">FAILURE</font>]&nbsp;</tt>")
                if self.indent > 0:
                    sys.stdout.write(2*self.indent*"&nbsp;")
                sys.stdout.write(str)
                if str[-1] == '\n':
                    sys.stdout.write("<br>")
            else:
                sys.stderr.write("[\033[1;31mFAILURE\033[0;39m] ")
                if self.indent > 0:
                    sys.stderr.write(self.indent*'  ')
                #;
                sys.stderr.write(str)
            #;
        #;
    #;

    def success(self,str):
        self.last = 2
        if self.verbosity > self.last:
            if self.html:
                sys.stdout.write("<tt>[<font color=\"#00ff00\">&nbsp;&nbsp;OK&nbsp;&nbsp;&nbsp;</font>]&nbsp</tt>")
                if self.indent > 0:
                    sys.stdout.write(2*self.indent*"&nbsp;")
                sys.stdout.write(str)
                if str[-1] == '\n':
                    sys.stdout.write("<br>")
            else:
                sys.stderr.write("[\033[1;32m  OK   \033[0;39m] ")
                if self.indent > 0:
                    sys.stderr.write(self.indent*'  ')
                #;
                sys.stderr.write(str)
            #;
        #;
    #;

    def warning(self,str):
        self.last = 1
        if self.verbosity > self.last:
            if self.html:
                sys.stdout.write("<tt>[<font color=\"#ffff00\">WARNING</font>]&nbsp;</tt>")
                if self.indent > 0:
                    sys.stdout.write(2*self.indent*"&nbsp;")
                #;
                sys.stdout.write(str)
                if str[-1] == '\n':
                    sys.stdout.write("<br>")
            else:
                sys.stderr.write("[\033[1;33mWARNING\033[0;39m] ")
                if self.indent > 0:
                    sys.stderr.write(self.indent*'  ')
                #;
                sys.stderr.write(str)
            #;
        #;
    #;

    def info(self,str):
        self.last = 2
        if self.verbosity > self.last:
            if self.html:
                sys.stdout.write("<tt>[<font color=\"#0000ff\">&nbsp;INFO&nbsp;&nbsp;</font>]&nbsp;</tt>")

                if self.indent > 0:
                    sys.stdout.write(2*self.indent*"&nbsp;")
                #;
                sys.stdout.write(str)
                if str[-1] == '\n':
                    sys.stdout.write("<br>")
            else:
                sys.stderr.write("[\033[1;34m INFO  \033[0;39m] ")
                if self.indent > 0:
                    sys.stderr.write(self.indent*'  ')
                #;
                sys.stderr.write(str)
            #;
    #;

    def debug(self,str):
        if self.debug_mode:
            if self.html:
                sys.stdout.write("<tt>[<font color=\"#ff00ff\">&nbsp;DEBUG&nbsp;</font>]&nbsp;</tt>")
                if self.indent > 0:
                    sys.stdout.write(2*self.indent*"&nbsp;")
                #;
                sys.stdout.write(str)
                if str[-1] == '\n':
                    sys.stdout.write("<br>")
            else:
                sys.stderr.write("[\033[1;35m DEBUG \033[0;39m] ")
                if self.indent > 0:
                    sys.stderr.write(self.indent*'  ')
                #;
                sys.stderr.write(str)
            #;
        #;
    #;

    def set_indent(self, i):
        self.indent = i
    #;

    def verbose(self,v):
        self.verbosity = v
    #;

    def append(self, str):
        if self.verbosity > self.last:
            if self.html:
                sys.stdout.write(str)
                if str[-1] == '\n':
                    sys.stdout.write("<br>")
            else:
                sys.stderr.write(str)
            #;
        #;
    #;
#;

if __name__ == "__main__":

    mess = Message()

    mess.info("hello, world\n")
    mess.set_indent(2)
    mess.info("hello, world\n")
    mess.html = True
    mess.info("hello, world\n")
####
