# -*- mode: python; mode: font-lock; py-indent-offset: 4; -*-
# $Id: mirror.py,v 1.5 2004/12/07 19:31:18 mok Exp $
#
# Copyright (C) 2004 Morten Kjeldgaard <mok@bioxray.dk>
#
# This file is part of the program "repo-janitor".
#
# "repo-janitor" is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# "repo-janitor" is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with "repo-janitor" in a file named "COPYING".  If not, visit
# their website at http://www.gnu.org for a copy, or request a copy by
# writing to:
#     Free Software Foundation, Inc.
#     59 Temple Place, Suite 330
#     Boston, MA 02111-1307 USA

# This class describes a directory a set of RPMs,
# including an external URL to a mirror.

import os
import urlparse, commands
import util

class Mirror:
    
    def __init__(self, msg=None):
        # pass an optional object to print messages
        self.url = ""                 # from
        self.path = ""                # to
        self.cmd = ""
        self.m = msg

    def __str__ (self):
        if self.url:
            return "%s --> %s" % (self.url,self.path)
        else:
            return ""
    #;

    def __repr__ (self):
        if self.url:
            return "%s --> %s" % (self.url,self.path)
        else:
            return ""
    #;

    # Define the source we will mirror. Depending on protocol, we
    # define different programs to deal with the situation.
    def set_url(self, s):

        self.url = s
        T = urlparse.urlparse(s)

        if T[0] == "http":
            self.cmd = "/usr/bin/wget -q -N -r -nd -l2 -P%(path)s -A.rpm %(url)s"
            
        elif T[0] == "ftp":

            if os.path.exists("/usr/bin/lftp"):
                self.cmd = "/usr/bin/lftp -c\'mirror -cL -i .*\.rpm$ -x .*-debuginfo-.*\.rpm$ %(url)s %(path)s\'"
            else:
                self.cmd = "wget -q -N -r -nd -l2 -P%(path)s -A.rpm %(url)s"
            #;
            
        elif T[0] == "rsync":
            # rsync is currently not supported properly by urlparse,
            # so I can't use the urljoin method to make sure we have a
            # slash at the end of the url.
            self.cmd = "/usr/bin/rsync -rptgoDuq %(url)s %(path)s"
            if self.url[-1] != '/':
                self.url = self.url+'/'
            #;
        #;

        elif not T[1]:
            #self.url[0] = "file"
            # The url is a local directory. We will mirror from there...
            self.cmd = None
        #;
    #;

    # Set the destination path. A few checks are made, ensuring the
    # path is a directory and that the user can write there.
    def set_path(self, s):
        import stat
        
        self.path = s
        if not os.path.exists(s):
            try:
                os.makedirs(s)
            except OSError, e:
                if self.m: self.m.failure ("%s: %s\n" % (e.strerror,s))
                raise "Mirror", "cannot create directory"
            #;
        else:
            # path exists, but can we write there??
            mode  =  os.stat(self.path)[stat.ST_MODE]
            if not stat.S_ISDIR(mode):
                if self.m: self.m.failure("not a directory: %s\n" % self.path)
                raise "Mirror", "cannot mirror to regular file"
            #;
            if not (mode & stat.S_IWUSR):
                if self.m:
                    self.m.failure("cannot write to this directory: %s\n" % self.path)
                #;
                raise "Mirror", "cannot write"
            #;
        #;
    #;

    # Simply set the command line
    def set_cmd(self, s):
        self.cmd = s.replace('\n','')
    #;

    # Print out the command to be executed. For debugging, mostly...
    def print_cmd(self):
        if self.m and self.cmd:
            self.m.info (self.cmd % {'path' : self.path, 'url': self.url}+'\n')
        #;
    #;

    # This method executes the command line defined for this mirror...
    def exe(self):

        if self.cmd:
            str = self.cmd % {'path' : self.path, 'url': self.url}
            status = commands.getstatusoutput(str)

            # check the result of the command
            if status[0] != 0:
                if self.m: self.m.failure (status[1]+'\n')
                return False
            #;
            return True
        else:
            util.recursive_copy(self.m, self.url, self.path)
        #;
    #; method exe
#; class Mirror


if __name__ == "__main__":
    import message

    msg = message.Message()

    m = Mirror(msg)
    #m.set_url("ftp://apt.bioxray.dk/pub/python/packages/Python2.1/RPMS")
    m.set_url("/u/mok/devel/repo-janitor")
    m.set_path("/tmp/xxx")
    m.print_cmd()
    m.exe()

####

