# -*- mode: python; mode: font-lock; py-indent-offset: 4; -*-
# $Id: parser.py,v 1.25 2004/12/07 19:34:36 mok Exp $
#
# Copyright (C) 2004 Morten Kjeldgaard <mok@bioxray.dk>
#
# This file is part of the program "repo-janitor".
#
# "repo-janitor" is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# "repo-janitor" is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with "repo-janitor" in a file named "COPYING".  If not, visit
# their website at http://www.gnu.org for a copy, or request a copy by
# writing to:
#     Free Software Foundation, Inc.
#     59 Temple Place, Suite 330
#     Boston, MA 02111-1307 USA

import os

from xml.sax import make_parser
from xml.sax.handler import ContentHandler
#import mx.Stack
import stack

from repository import Repository
from distribution import Distribution
from release import Release
from component import Component
import message

gRepo = []

class ConfigParser(ContentHandler):

    def __init__(self):
        self.level = 0
#        self.state = mx.Stack.Stack()
#        self.stack = mx.Stack.Stack()
        self.state = stack.Stack()
        self.stack = stack.Stack()
        self.mirror_url = ''
        self.mirror_cmd = ''

    #;
    
    def startElement(self, name, attrs):
        self.level = self.level+1
        m = message.Message()

        if name == "repo-janitor":
            if attrs.has_key("id"):
                id = attrs["id"]
            else:
                id = "Apt/Yum enabled RPM Repository"
            #;
            if not attrs.has_key("topdir"):
                m.failure ("Can't continue without topdir!\n")
                m.info ("It's defined in the <repo-janitor topdir=\"/some/place\"> tag\n")
                m.info ("in the XML setup file!\n")
                raise SystemExit
            #;

            self.obj = Repository()
            self.obj.set_id(id)
            self.obj.set_path(attrs["topdir"])

        elif name == "distribution":
            if attrs.has_key("id"):
                id = attrs["id"]
            else:
                id = "fedora"
            #;

            self.parent = self.obj
            path = self.parent.path
            self.stack.push(self.parent)

            self.obj = Distribution()
            self.obj.set_id(id)
            self.obj.set_path(os.path.join(path,id))
            self.obj.topdir = path
            self.obj.set_local(self.parent.local)
            self.obj.ignore = self.parent.ignore
            self.obj.set_parent(self.parent)  # let this dist know repo
            self.parent.add(self.obj)   # let repo know this dist
            self.obj.repo = self.parent
            self.obj.relpath = id

        elif name == "release":
            if attrs.has_key("id"):
                id = attrs["id"]
            else:
                id = "fc1"
            #;
            if attrs.has_key("arch"):
                arch = attrs["arch"]
            else:
                arch = "i386"
            #;
            self.parent = self.obj
            path = self.parent.path
            self.stack.push(self.parent)

            self.obj = Release()
            self.obj.set_id(id)
            self.obj.set_arch(arch)
            self.obj.set_path(os.path.join(path,id))
            self.obj.set_local(self.parent.local)
            self.obj.ignore = self.parent.ignore
            self.obj.set_parent(self.parent) # let rel know dist
            self.parent.add(self.obj)        # let dist know about rel
            self.obj.repo = self.parent.repo
            self.obj.relpath = os.path.join(self.obj.parent.relpath,id)

        elif name == "component":
            if attrs.has_key("id"):
                id = attrs["id"]
            else:
                id = "core"
            #;
            if id == "base":
                m.failure("component cannot be named \"base\", choose another name\n")
                raise SystemExit
            #;
            self.parent = self.obj
            path = os.path.join(self.parent.path,self.parent.arch)
            self.stack.push(self.parent)

            self.obj = Component()
            self.obj.set_path(path)
            self.obj.set_id(id)
            self.obj.set_local(self.parent.local)
            self.obj.ignore = self.parent.ignore
            self.obj.set_parent(self.parent) # let comp know about rel
            self.parent.add(self.obj)        # let rel know about comp
            self.obj.repo = self.parent.repo
            self.obj.set_arch(self.parent.arch)
            self.obj.relpath = os.path.join(self.parent.relpath,self.parent.arch)

        elif name == "local":
            self.obj.set_local(True)

        elif name == "allow":
            self.state.push(1)

        elif name == "mirror":
            if attrs.has_key("type"):
                if attrs["type"] == "rpm":
                    self.state.push(2)
                elif attrs["type"] == "srpm":
                    self.state.push(3)
                else:
                    m.failure("mirror type must have value rpm or srpm\n")
                    raise SystemExit
                #;
            else:
                m.failure("missing type attribute in mirror tag\n")
                raise SystemExit
            #;

        elif name == "cmd":
            self.state.push(5)
            
        elif name == "description":
            self.state.push(4)
            if attrs.has_key("file"):
                f = os.path.expandvars(attrs["file"])
                if os.path.exists(f):
                    self.obj.dfile = f
                else:
                    m.warning("description file %s not found, therefore ignored\n" % f)
                    self.obj.dfile = None
                #;
            #;
        elif name == "ignore":
            self.obj.ignore = True

        elif name == "maintainer":
            if attrs.has_key("name"):
                self.obj.maintainer = attrs["name"]
            else:
                self.obj.maintainer = "Anonymous Coward" #  hommage to /.
            #;
            if attrs.has_key("email"):
                self.obj.email = attrs["email"]
            else:
                self.obj.email = "root@"+os.uname()[1]
            #;
        elif name == "url":
            if attrs.has_key("href"):
                self.obj.url = attrs["href"]
            else:
                m.warning("url tag ignored, must have href attribute\n");
            #;
        elif name == "stylesheet":
            if attrs.has_key("href"):
                self.obj.stylesheet = attrs["href"]
            else:
                m.warning("stylesheet tag ignored, must have href attribute\n");
            #;
        #;
        return
    #;

    def endElement(self, name):
        # Handle end tag
        import mirror
        global gRepo
        
        self.level = self.level-1
        if name == "release":
            self.obj = self.stack.pop()

        elif name == "component":
            self.obj = self.stack.pop()

        elif name == "distribution":
            self.obj = self.stack.pop()
            
        elif name == "repo-janitor":
            gRepo.append(self.obj)

        elif name == "mirror":

            state = self.state.pop()

            if state == 2:                  # We are inside mirror tags (type RPMS)
                mi = mirror.Mirror(self.obj.rpmdir.m)
                mi.set_url(self.mirror_url.strip())
                mi.set_path(self.obj.rpmdir.path)
                if self.mirror_cmd: mi.set_cmd(self.mirror_cmd.strip())
                self.obj.rpmdir.mirrors.append(mi)

            elif state == 3:                  # We are inside mirror tags (type SRPMS)
                mi = mirror.Mirror(self.obj.srpmdir.m)
                mi.set_url(self.mirror_url.strip())
                mi.set_path(self.obj.srpmdir.path)
                if self.mirror_cmd: mi.set_cmd(self.mirror_cmd.strip())
                self.obj.srpmdir.mirrors.append(mi)
            #;

            self.mirror_url = ''
            self.mirror_cmd = ''

        elif name == "cmd":
            self.state.pop()

        elif name == "description":
            self.state.pop()
        #;
        
        return
    #;


    def characters(self,ch):
        # Handle data between tags

        #if not self.state:
        if self.state.isempty():
            return
        #;
        if not ch:
            return
        #;
        
        #state = self.state[-1]          # mxStack has no top() method...
        state = self.state.top()

        if state == 2 or state == 3:
            self.mirror_url += ch 

        elif state == 4:                  # We are inside description tags
            self.obj.description += ch

        elif state == 5:                  # We are inside mirror and cmd tags
            self.mirror_cmd += ch
        #;
        return
    #;
#;

####
