# -*- mode: python; mode: font-lock; py-indent-offset: 4; -*-
# $Id: rpmfile2.py,v 1.10 2004/09/28 20:23:31 mok Exp $
#
# Copyright (C) 2004 Morten Kjeldgaard <mok@bioxray.dk>
#
# This file is part of the program "repo-janitor".
#
# "repo-janitor" is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# "repo-janitor" is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with "repo-janitor" in a file named "COPYING".  If not, visit
# their website at http://www.gnu.org for a copy, or request a copy by
# writing to:
#     Free Software Foundation, Inc.
#     59 Temple Place, Suite 330
#     Boston, MA 02111-1307 USA

import gzip
import UserDict
import os
import rpm
RPMerror = "RPM error"

# Python classes that gives us access to all the data stored in an RPM
# file or an RPM database entry.The resulting object can be treated
# like a dictionary, with rpm tags as keys. First version mok@bioxray.dk 990706
#
# Note: since Python is white-space-sensitive and already contains a
# begin-block mark (it is the colon) my Python programming style
# includes an "end-of-brace" marked by #; Thus, if this program is
# ever scrambled, it is possible to regenerate it with a script.

class RPMfile(UserDict.UserDict):
    # This class represents an RPM file The resulting object can be
    # treated like a dictionary, with rpm tags as keys.

    def __init__(self, file):

        self.rpmfile = file

    def open(self):
        ts = rpm.TransactionSet("", rpm._RPMVSF_NOSIGNATURES)
        fd = os.open (self.rpmfile, os.O_RDONLY)
        try:
            hdr = ts.hdrFromFdno(fd)
            self.data = hdr
        except rpm.error, rpm.rpmts_HdrFromFdno:
            self.data = None
            raise RPMerror, "problem with RPM file"
        #;
        os.close(fd)
    #;

    # write data to a cache
    def write(self,cachefile):
        fd=gzip.open(cachefile, "w")
        fd.write(self.data.unload(1))
        fd.close()
    #;

    # load data from a cached file
    def read(self,cachefile):
        try:
            fd=gzip.open(cachefile)
            self.data = rpm.headerLoad(fd.read())
            fd.close()
        except IOError:
            self.open()
            self.write(cachefile)
        #; 
        return self
    #;
 
    def info (self):
        # Give us the whole info scoop from the file"
        import time
        info = """
 Name        : %-28s Relocations: %s
 Version     : %-33s Vendor: %s
 Release     : %-29s Build Date: %s        
 Install Date: %-29s Build Host: %s
 Group       : %-29s Source RPM: %s
 Size        : %-32s License: %s
 Signature   : %s 
 Packager    : %s 
 URL         : %s 
 Summary     : %s 
 Description : %s""" % \
        (self.data["NAME"], self.data["PREFIXES"], self.data["VERSION"],
         self.data["VENDOR"], self.data["RELEASE"], time.ctime(self.data["BUILDTIME"]),
         self.data["INSTALLTIME"], self.data["BUILDHOST"],self.data["GROUP"],
         self.data["SOURCERPM"], self.data["SIZE"], self.data["LICENSE"],
         self.data["HEADERSIGNATURES"],self.data["PACKAGER"], self.data["URL"],
         self.data["SUMMARY"], self.data["DESCRIPTION"])

        return info
    #;
    def __str__(self):
        return self.info()
    #;

    def filelist (self):
        # Give us the list of files in the package
        list = self.data["FILENAMES"]
        return list
    #;

    # Return the ident of the package in Yum convention (nevra)
    # name-epoch-version-release.arch. Fmt == 1 is for the header.info file.

    def get_id (self,fmt=0):
        from string import join

        epoch = self.data["EPOCH"]
        if epoch is None:
            epoch = 0
        #;
        epoch = str(epoch)
        if fmt == 1:
            s = join([self.data["NAME"],self.data["VERSION"],self.data["RELEASE"]],'-')
            s = "%s:%s" % (epoch,s)
        else:
            s = join([self.data["NAME"],epoch,self.data["VERSION"],self.data["RELEASE"]],'-')
        #;

        if self.data["SOURCEPACKAGE"]:
            return join([s,"src"], '.')
        else:
            return join([s,self.data["ARCH"]], '.')
        #;
    #;

    def is_source(self):
        if self.data["SOURCEPACKAGE"]:
            return True
        else:
            return False
        #;
    #;
    
#;

if __name__ == "__main__":
    import sys

    if len(sys.argv) < 2:
        print "Usage: rpmfile2.py <rpmfile>"
        raise SystemExit
    #;
    
    try:
        x = RPMfile(sys.argv[1])
        x.open()
    except RPMerror:
        print "aborting..."
        raise SystemExit
    #;
    
    print "Name:", x["NAME"]
    print "Version:", x["VERSION"]
    print "Release:", x["RELEASE"]
    print "Architecture:", x["ARCH"]
    print "List of files:"
    for i in x.filelist():
        print i
    #;
    print x.info()
    print x.get_id(1)
    print x.keys()
    
#;

####
