# -*- mode: python; mode: font-lock; py-indent-offset: 4; -*-
# $Id: util.py,v 1.20 2004/09/26 12:55:14 mok Exp $
#
# Copyright (C) 2004 Morten Kjeldgaard <mok@bioxray.dk>
#
# This file is part of the program "repo-janitor".
#
# "repo-janitor" is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# "repo-janitor" is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with "repo-janitor" in a file named "COPYING".  If not, visit
# their website at http://www.gnu.org for a copy, or request a copy by
# writing to:
#     Free Software Foundation, Inc.
#     59 Temple Place, Suite 330
#     Boston, MA 02111-1307 USA

# Utility functions for repo-janitor

import os, stat
import sys
import buildenv

# Set the permission of the files in the list to 'mode'

def chmod (ff, mode, msg = None):
    from stat import ST_MODE, S_IMODE
    import message

    if not os.path.exists(ff) or os.path.islink(ff):
        return
               
    # File is there, check the protection
    st = os.stat(ff)[ST_MODE]
    m = S_IMODE(st)
    if m != mode:
        os.chmod(ff, mode)
        if msg:
            msg.warning("mode change %o -> %o %s\n" % (m,mode,os.path.basename(ff)))
        #;
    #;
    return
#;


# Create a .htaccess file in the given directory
def create_htaccess(path,allow):
    if not allow or not os.path.exists(path):
        return
    f = open(os.path.join(path,".htaccess"),"w")
    f.write("<Limit GET>\n")
    f.write("  order deny,allow\n")
    f.write("  deny from all\n")
    for a in allow:
        f.write("  allow from %s\n" % a)
    #;
    f.write("</Limit>\n")
    f.close()
#;


# Create an rpmmacros file for building a given component
def create_rpmmacros(path, rpmdir, srpmdir):
    import pwd
    
    f = open(path,"w")
    f.write("%%_topdir %s\n" % buildenv.topdir)
    f.write("%_sourcedir %{_topdir}/SOURCES/%{name}\n")
    f.write("%_specdir %{_topdir}/SPECS\n")
    f.write("%_builddir %{_topdir}/BUILD\n")
    f.write("%_tmppath %{_topdir}/tmp\n")
    f.write("%%_rpmdir %s\n" % rpmdir)
    f.write("%%_srpmdir %s\n" % srpmdir)
    f.write("%_rpmfilename %%{NAME}-%%{VERSION}-%%{RELEASE}.%%{ARCH}.rpm\n")
    uid = os.getuid()
    f.write("%%packager %s\n" % pwd.getpwuid(uid)[4])
    f.write("%vendor repo-janitor\n")
    f.close()
#;

# Create the release file for apt. I don't really know what information
# is supposed to be in there, it doesn't seem to matter...
def create_release(f, arch, comps):
    import time

    f.write("Origin: Unknown\nLabel: Unknown\nSuite: Unknown\n")
    f.write("Codename: Unknown\nDate: %s\n" % time.ctime(time.time()))
    f.write ("Architectures: %s\n" % arch)
    s = "Components: "
    for comp in comps:
        s = s+comp.id+" "
    #;
    f.write (s)
    f.write ("\nDescription: Not available\nMD5Sum:\n")
    chmod (f.name,0644)
#;

# Michael Stenner's lock()/unlock() is gratefully lifted from yum

def lock(filename, contents='', mode=0777):
    try:
        fd = os.open(filename, os.O_EXCL|os.O_CREAT|os.O_WRONLY, mode)
    except OSError, msg:
        if not msg.strerror == "File exists": raise msg
        return 0
    else:
        os.write(fd, contents)
        os.close(fd)
        return 1
    #;
#;

def unlock(filename):
    try:
        os.unlink(filename)
    except OSError, msg:
        pass
    #;
#;


# Process a string with HTML encodings to defeat address spiders.
# Gratefully lifted from HTMLgen.py (where it is private to class MailTo)

def antispam(address):
    from whrandom import choice
    import string
    
    buffer = map(None, address)
    for i in range(0, len(address), choice((2,3,4))):
        buffer[i] = '&#%d;' % ord(buffer[i])
    return string.join(buffer,'')


# Receive a list of files and directories, and zap them without mercy.

def delete(msg, death_row):

    if not death_row:
        return
    for f,reason in death_row:
        if not os.path.isdir(f):
            try:
                os.unlink(f)
                if msg: msg.success("deleted %s, %s\n" %(f,reason))
            except OSError, e:
                if msg: msg.failure("%s: %s\n" % (e.strerror,f))
            #;
        else:
            try:
                os.path.walk(f,delete_dir,msg)
            except OSError:
                if msg: msg.failure("remove directory manually: %s\n" %f)
            #;
        #;
    return
    #;

# The "walk" function, called by "delete".

def delete_dir(msg, dirname, files):

    # prepend directory to all file names
    L = map(lambda x: os.path.join(dirname,x), files)

    # convert file list to a list of tuples with a reason
    reason="not owned by me"
    delete (msg, map (lambda x: tuple([x,reason]),L))

    try:
        os.rmdir (dirname)
        if msg: msg.success("deleted directory %s, %s\n" %(dirname,reason))        
    except OSError:
        if msg: msg.failure("in delete_dir")
    #;
    return
#;


# See which of two files (directories) is newer. Return 1 if
# 1 is newer, -1 if 2 is newer, 0 if equal.

def isnewer(file1, file2):

    try:
        st1 = os.stat(file1)
        age1 = st1[stat.ST_MTIME]
    except OSError:
        age1 = 0
    #;
    try:
        st2 = os.stat(file2)
        age2 = st2[stat.ST_MTIME]
    except OSError:
        age2 = 0
    #;

    if age1 > age2:
        return 1
    if age1 < age2:
        return -1
    return 0
#;

# Some rogue characters make htmlgen croak. Actually, the rpm interface is in error,
# because the description returns an ascii string, even if it contains characters >127.
# So we simply convert those characters to html escape sequences.

def fixrogue(s):
    plain = ''
    for i in s:
        if ord(i) > 127:
            plain += "&#%d;" % ord(i)
        else:
            plain += str(i)
        #;
    #;
    return plain
#;

# Recursively copy a directory tree. If possible, create a hard link instead
# of a copy. Used for local mirroring (class rpmdir).
def recursive_copy(msg, fromdir, todir):
    import shutil

    if not os.path.isdir(fromdir):
        if msg: msg.failure("%s is not a directory, correct in config file\n" % fromdir)
        return
    #;

    try:
        K = os.listdir(fromdir)
    except OSError, e:
        if msg: msg.failure("%s: %s\n" % (e.strerror,fromdir))
        return
    #;

    for f in K:
        ff = os.path.join(fromdir,f)
        tt = os.path.join(todir,f)

        if os.path.isdir(ff):
            # source is a directory. Descend into it with this function.
            try:
                os.makedirs(tt)
            except OSError:
                # makedirs throws an exeption if the directory already exists,
                # (but we ignore that) or if the directory can't be created (but
                # we already have checked for that)
                pass

            # call myself again to copy sub-directories.
            recursive_copy(msg, ff, tt)

        else:
            # source is a file, create link or copy. 
            if not os.path.exists(tt) or isnewer(ff,tt):
                try:
                    os.link (os.path.realpath(ff),os.path.realpath(tt))
                except OSError:
                    shutil.copy2 (os.path.realpath(ff),os.path.realpath(tt))
                #;
            #;
        #;
    #;
#;


# This function creates metadata for yum version 2.2 and higher, but only if
# createrepo has been installed.
def generate_metadata(msg, dir):
    # Stock createrepo does not work as a module
    #crdir = "/usr/share/createrepo"
    #if not crdir in sys.path:
    #    sys.path.append(crdir)
    #;

    try:
        import genpkgmetadata
    except ImportError:
        # out of luck :-(
        msg.failure ("OH NO!\n")
        return
    #;

    # Launch createrepo in quiet mode
    msg.info ("Creating metadata...")
    L = ["-q", dir]
    genpkgmetadata.main(L)
    msg.append("done.\n")

    return
#;
    
####
