# -*- mode: python; mode: font-lock; py-indent-offset: 4; -*-
# $Id: webgen.py,v 1.6 2004/10/08 13:33:19 mok Exp $
#
# Copyright (C) 2004 Morten Kjeldgaard <mok@bioxray.dk>
#
# This file is part of the program "repo-janitor".
#
# "repo-janitor" is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# "repo-janitor" is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with "repo-janitor" in a file named "COPYING".  If not, visit
# their website at http://www.gnu.org for a copy, or request a copy by
# writing to:
#     Free Software Foundation, Inc.
#     59 Temple Place, Suite 330
#     Boston, MA 02111-1307 USA

import os, time
import HTMLgen
import util
import version

template = 0                            # Set if HTML templates wanted

class _MyDocument(HTMLgen.SimpleDocument):

    def __init__(self, obj, resource = None, **kw):

        HTMLgen.SimpleDocument.__init__(self, resource, **kw)

        self.title = obj.repo.id
        self.author = obj.repo.maintainer
        self.email = obj.repo.email
        self.suffix = ".html"

        self.subst = {"repository":HTMLgen.Href(obj.repo.url,obj.repo.id),
                 "date":time.strftime("%d %b %Y %T",time.localtime()),
                 "time":time.strftime("%R",time.localtime()),
                 "weekday":time.strftime("%A",time.localtime()),
                 "day":time.strftime("%d",time.localtime()),
                 "month":time.strftime("%B",time.localtime()),
                 "year":time.strftime("%Y",time.localtime()),
                 "maintainer":obj.repo.maintainer,
                 "email":util.antispam(obj.repo.email),
                 "host":os.uname()[1],
                 "url":util.antispam(obj.repo.url),
                 "version":version.__version__,
                 "rj": HTMLgen.Href(obj.repo.rj,"repo-janitor")}

        if obj.type == 3:
            self.subst["distribution"] = obj.parent.parent.id,
            self.subst["release"] = obj.parent.id
            self.subst["arch"] = obj.arch
            self.subst["component"] = obj.id
        elif obj.type == 2:
            self.subst["distribution"] = obj.parent.id,
            self.subst["release"] = obj.id
            self.subst["arch"] = obj.arch
        elif obj.type == 1:
            self.subst["distribution"] = obj.id
        #;
    #;

    def header(self, str):
        self.append( HTMLgen.Heading(4, str, Class="header") )
        return
    #;

    # Generate the standard footer markups.
    def footer(self):
        span = HTMLgen.Span(Class="footer")
        s = "<p><hr>Date: {date}<br>Maintainer: {maintainer}<br>Generated by: {rj} {version}<hr>"
        span.append(HTMLgen.StringTemplate(s, self.subst))
        self.append(span)
    #;

    # Add content to the Document object, but pass through
    # substitution filter

    def append_subst (self,s):
        self.append(HTMLgen.StringTemplate(s, self.subst))
        return
    #;
#;

# This class is used if the generated HTML is to be included in
# a template defining the layout of the site.

class _MyTemplateDocument(_MyDocument):

    def __init__(self, obj, resource = None, **kw):
        MyDocument.__init__(self, obj, resource = None, **kw)
        self.suffix = ".ht"
        return
    #;

    def __str__(self):
        import string

        s = []
        s.append('\n<!-- This file generated using Python HTMLgen module. -->\n')
        # DOCUMENT CONTENT SECTION and FOOTER added on
        bodystring = '%s\n' * len(self.contents)
        s.append((bodystring % tuple(self.contents)))

        return string.join(s, '')
    #;
#;


# Generator function, returns whatever class caller wants
def Document():
    if template == 0:
        return _MyDocument
    else:
        return _MyTemplateDocument
    #;
#;
####
